@@ -1,18 +1,22 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 0x1777
 #        2016-17 Nick Boultbee
 #           2017 Didier Villevalois
 #           2017 Muges
 #           2017 Eyenseo
+#           2018 Joschua Gandert
+#           2018 Blimmo
+#           2018 Olli Helin
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
+from math import ceil, floor
+from typing import List
+
 from gi.repository import Gtk, Gdk, Gst
 import cairo
-from math import ceil, floor
 
 from quodlibet import _, app
 from quodlibet import print_w
@@ -26,13 +30,14 @@ from quodlibet.qltk.seekbutton import TimeLabel
 from quodlibet.qltk.tracker import TimeTracker
 from quodlibet.qltk import get_fg_highlight_color
 from quodlibet.util import connect_destroy, print_d
+from quodlibet.util.path import uri2gsturi
 
 
 class WaveformSeekBar(Gtk.Box):
     """A widget containing labels and the seekbar."""
 
     def __init__(self, player, library):
-        super(WaveformSeekBar, self).__init__()
+        super().__init__()
 
         self._player = player
         self._rms_vals = []
@@ -48,6 +53,7 @@ class WaveformSeekBar(Gtk.Box):
 
         for child in self.get_children():
             child.show_all()
+        self.set_time_label_visibility(CONFIG.show_time_labels)
 
         self._waveform_scale.connect('size-allocate',
                                      self._update_redraw_interval)
@@ -74,6 +80,15 @@ class WaveformSeekBar(Gtk.Box):
         if player.info:
             self._create_waveform(player.info, CONFIG.max_data_points)
 
+    def set_time_label_visibility(self, is_visible):
+        self._time_labels_visible = is_visible
+        if is_visible:
+            self._elapsed_label.show()
+            self._remaining_label.show()
+        else:
+            self._elapsed_label.hide()
+            self._remaining_label.hide()
+
     def _create_waveform(self, song, points):
         # Close any existing pipeline to avoid leaks
         self._clean_pipeline()
@@ -93,7 +108,7 @@ class WaveformSeekBar(Gtk.Box):
 
         command = command_template.format(interval)
         pipeline = Gst.parse_launch(command)
-        pipeline.get_by_name("uridec").set_property("uri", song("~uri"))
+        pipeline.get_by_name("uridec").set_property("uri", uri2gsturi(song("~uri")))
 
         bus = pipeline.get_bus()
         self._bus_id = bus.connect("message", self._on_bus_message, points)
@@ -204,6 +219,10 @@ class WaveformSeekBar(Gtk.Box):
         self._update_waveform(player, full_redraw)
 
     def _update_label(self, player):
+        if not self._time_labels_visible:
+            self.set_sensitive(player.info is not None and player.seekable)
+            return
+
         if player.info:
             if self._hovering:
                 # Show the position pointed by the mouse
@@ -293,12 +312,12 @@ class WaveformSeekBar(Gtk.Box):
 class WaveformScale(Gtk.EventBox):
     """The waveform widget."""
 
-    _rms_vals = []
+    _rms_vals: List[int] = []
     _player = None
     _placeholder = True
 
     def __init__(self, player):
-        super(WaveformScale, self).__init__()
+        super().__init__()
         self._player = player
         self.set_size_request(40, 40)
         self.position = 0
@@ -308,7 +327,8 @@ class WaveformScale(Gtk.EventBox):
 
         self.mouse_position = -1
         self._last_mouse_position = -1
-        self.add_events(Gdk.EventMask.POINTER_MOTION_MASK)
+        self.add_events(Gdk.EventMask.POINTER_MOTION_MASK |
+                        Gdk.EventMask.SCROLL_MASK)
 
         self._seeking = False
 
@@ -333,6 +353,14 @@ class WaveformScale(Gtk.EventBox):
 
         # Compute the coarsest time interval for redraws
         length = self._player.info("~#length")
+        if (length == 0):
+            # The length is 0 for example when playing a stream from
+            # Internet radio. If 0 is passed forward as the update interval,
+            # UI will freeze as it will try to update continuously.
+            # The update interval is usually 1 second so use that instead.
+            print_d("Length is zero for %s, using redraw interval of 1000 ms"
+                % self._player.info)
+            return 1000
         return length * 1000 / max(width * pixel_ratio, 1)
 
     def compute_redraw_area(self):
@@ -534,6 +562,14 @@ class WaveformScale(Gtk.EventBox):
             self.queue_draw()
             return True
 
+    def do_scroll_event(self, event):
+        if event.direction == Gdk.ScrollDirection.UP:
+            self._player.seek(self._player.get_position() + CONFIG.seek_amount)
+            self.queue_draw()
+        elif event.direction == Gdk.ScrollDirection.DOWN:
+            self._player.seek(self._player.get_position() - CONFIG.seek_amount)
+            self.queue_draw()
+
     def set_position(self, position):
         self.position = position
 
@@ -548,14 +584,17 @@ class WaveformScale(Gtk.EventBox):
         return ratio * length
 
 
-class Config(object):
+class Config:
     _config = PluginConfig(__name__)
 
     elapsed_color = ConfProp(_config, "elapsed_color", "")
     hover_color = ConfProp(_config, "hover_color", "")
     remaining_color = ConfProp(_config, "remaining_color", "")
     show_current_pos = BoolConfProp(_config, "show_current_pos", False)
+    seek_amount = IntConfProp(_config, "seek_amount", 5000)
     max_data_points = IntConfProp(_config, "max_data_points", 3000)
+    show_time_labels = BoolConfProp(_config, "show_time_labels", True)
+
 
 CONFIG = Config()
 
@@ -570,6 +609,9 @@ class WaveformSeekBarPlugin(EventPlugin):
     PLUGIN_DESC = _(
         "A seekbar in the shape of the waveform of the current song.")
 
+    def __init__(self):
+        self._bar = None
+
     def enabled(self):
         self._bar = WaveformSeekBar(app.player, app.librarian)
         self._bar.show()
@@ -578,7 +620,7 @@ class WaveformSeekBarPlugin(EventPlugin):
     def disabled(self):
         app.window.set_seekbar_widget(None)
         self._bar.destroy()
-        del self._bar
+        self._bar = None
 
     def PluginPreferences(self, parent):
         red = Gdk.RGBA()
@@ -611,8 +653,17 @@ class WaveformSeekBarPlugin(EventPlugin):
 
         def on_show_pos_toggled(button, *args):
             CONFIG.show_current_pos = button.get_active()
+
+        def seek_amount_changed(spinbox):
+            CONFIG.seek_amount = spinbox.get_value_as_int()
+
         vbox = Gtk.VBox(spacing=6)
 
+        def on_show_time_labels_toggled(button, *args):
+            CONFIG.show_time_labels = button.get_active()
+            if self._bar is not None:
+                self._bar.set_time_label_visibility(CONFIG.show_time_labels)
+
         def create_color(label_text, color, callback):
             hbox = Gtk.HBox(spacing=6)
             hbox.set_border_width(6)
@@ -642,4 +693,24 @@ class WaveformSeekBarPlugin(EventPlugin):
         show_current_pos.connect("toggled", on_show_pos_toggled)
         vbox.pack_start(show_current_pos, True, True, 0)
 
+        show_time_labels = Gtk.CheckButton(label=_("Show time labels"))
+        show_time_labels.set_active(CONFIG.show_time_labels)
+        show_time_labels.connect("toggled", on_show_time_labels_toggled)
+        vbox.pack_start(show_time_labels, True, True, 0)
+
+        hbox = Gtk.HBox(spacing=6)
+        hbox.set_border_width(6)
+        label = Gtk.Label(label=_(
+            "Seek amount when scrolling (milliseconds):"
+        ))
+        hbox.pack_start(label, False, True, 0)
+        seek_amount = Gtk.SpinButton(
+            adjustment=Gtk.Adjustment(CONFIG.seek_amount,
+                                      0, 60000, 1000, 1000, 0)
+        )
+        seek_amount.set_numeric(True)
+        seek_amount.connect("changed", seek_amount_changed)
+        hbox.pack_start(seek_amount, True, True, 0)
+        vbox.pack_start(hbox, True, True, 0)
+
         return vbox
