@@ -51,7 +51,7 @@ class CoverSourcePlugin(GObject.Object):
     def __init__(self, song, cancellable=None):
         self.song = song
         self.cancellable = cancellable
-        super(CoverSourcePlugin, self).__init__()
+        super().__init__()
 
     @classmethod
     def group_by(cls, song):
@@ -140,7 +140,7 @@ class CoverSourcePlugin(GObject.Object):
         try:
             return open(cp, 'rb') if cp and path.isfile(cp) else None
         except IOError:
-            print_w('Failed reading album art "%s"'.format(path))
+            print_w('Failed reading album art "{}"'.format(path))
 
     def search(self):
         """
