@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2012 Joe Wreschnig, Michael Urman, Iñigo Serna
-#           2011-2017 Nick Boultbee
+#           2011-2020 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -8,39 +7,38 @@
 # (at your option) any later version.
 
 import sys
+from typing import Optional, Type, Sequence
 
 from gi.repository import Gtk, Pango, Gdk
 
-from quodlibet import C_, _
-from quodlibet import qltk
+from quodlibet import C_, _, print_e, print_d
+from quodlibet import app
 from quodlibet import config
+from quodlibet import qltk
 from quodlibet import util
-
-from quodlibet.util import massagers
-
 from quodlibet.formats import AudioFileError
-from quodlibet.qltk.completion import LibraryValueCompletion
-from quodlibet.qltk.tagscombobox import TagsComboBox, TagsComboBoxEntry
-from quodlibet.qltk.views import RCMHintedTreeView, TreeViewColumn
-from quodlibet.qltk.wlw import WritingWindow
-from quodlibet.qltk.window import Dialog
-from quodlibet.qltk.models import ObjectStore
-from quodlibet.qltk.ccb import ConfigCheckButton
-from quodlibet.qltk.x import SeparatorMenuItem, Button, MenuItem
+from quodlibet.plugins import PluginManager
+from quodlibet.plugins.editing import EditTagsPlugin
+from quodlibet.qltk import Icons
 from quodlibet.qltk._editutils import EditingPluginHandler, OverwriteWarning
 from quodlibet.qltk._editutils import WriteFailedError
-from quodlibet.qltk import Icons
-from quodlibet.plugins import PluginManager
+from quodlibet.qltk.ccb import ConfigCheckButton
+from quodlibet.qltk.completion import LibraryValueCompletion
+from quodlibet.qltk.models import ObjectStore
+from quodlibet.qltk.tagscombobox import TagsComboBox, TagsComboBoxEntry
+from quodlibet.qltk.views import RCMHintedTreeView, TreeViewColumn, BaseView
+from quodlibet.qltk.window import Dialog
+from quodlibet.qltk.wlw import WritingWindow
+from quodlibet.qltk.x import SeparatorMenuItem, Button, MenuItem
 from quodlibet.util import connect_obj
+from quodlibet.util import massagers
 from quodlibet.util.i18n import numeric_phrase
-from quodlibet.util.tags import USER_TAGS, MACHINE_TAGS, sortkey as tagsortkey
 from quodlibet.util.string.splitters import (split_value, split_title,
-    split_people, split_album)
-from quodlibet.compat import iteritems, string_types, text_type, listkeys, \
-    listmap, itervalues
+                                             split_people, split_album)
+from quodlibet.util.tags import USER_TAGS, MACHINE_TAGS, sortkey as tagsortkey
 
 
-class Comment(object):
+class Comment:
     """A summary of a collection of values for one tag"""
 
     def __init__(self, text, have=1, total=1, shared=True):
@@ -83,6 +81,11 @@ class Comment(object):
     def is_missing(self):
         return not self.complete
 
+    def get_shared_text(self):
+        if self.shared:
+            return util.escape(self.text)
+        return ''
+
     def get_markup(self):
         """Returns pango markup for displaying"""
 
@@ -152,10 +155,10 @@ class AudioFileGroup(dict):
         self._can_change = can_change
 
         # collect comment representations
-        for tag, count in iteritems(keys):
+        for tag, count in keys.items():
             first_value = first[tag]
-            if not isinstance(first_value, string_types):
-                first_value = text_type(first_value)
+            if not isinstance(first_value, str):
+                first_value = str(first_value)
             shared = all[tag]
             complete = count == total
             if shared and complete:
@@ -184,74 +187,113 @@ class AudioFileGroup(dict):
         return all(song.can_change(key) for song in self.songs)
 
 
-class SplitValues(Gtk.ImageMenuItem):
-    tags = False
-    needs = []
-    _order = 0.0
+class TagSplitter(EditTagsPlugin):
+    """Splits tag values into other tags"""
+    pass
+
+
+class SplitValues(TagSplitter):
 
     def __init__(self, tag, value):
-        super(SplitValues, self).__init__(
+        super().__init__(
             label=_("Split into _Multiple Values"), use_underline=True)
         self.set_image(Gtk.Image.new_from_icon_name(
             Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
         spls = config.gettext("editing", "split_on").split()
-        self.set_sensitive(len(split_value(value, spls)) > 1)
+        vals = [val if len(val) <= 64 else val[:64] + "…"
+                for val in split_value(value, spls)]
+        string = ", ".join(["{}={}".format(tag, val) for val in vals])
+        self.set_label(string)
+        self.set_sensitive(len(vals) > 1)
 
     def activated(self, tag, value):
         spls = config.gettext("editing", "split_on").split()
         return [(tag, v) for v in split_value(value, spls)]
 
 
-class SplitDisc(Gtk.ImageMenuItem):
+class SplitDisc(TagSplitter):
     tags = ["album"]
     needs = ["discnumber"]
     _order = 0.5
 
     def __init__(self, tag, value):
-        super(SplitDisc, self).__init__(
+        super().__init__(
             label=_("Split Disc out of _Album"), use_underline=True)
         self.set_image(Gtk.Image.new_from_icon_name(
             Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
-        self.set_sensitive(split_album(value)[1] is not None)
+
+        album, disc = split_album(value)
+        if disc is not None:
+            album = album if len(album) <= 64 else album[:64] + "…"
+            self.set_label("{}={}, {}={}".format(tag, album,
+                                                 self.needs[0], disc))
+
+        self.set_sensitive(disc is not None)
 
     def activated(self, tag, value):
         album, disc = split_album(value)
         return [(tag, album), ("discnumber", disc)]
 
 
-class SplitTitle(Gtk.ImageMenuItem):
+class SplitTitle(TagSplitter):
     tags = ["title"]
     needs = ["version"]
     _order = 0.5
 
     def __init__(self, tag, value):
-        super(SplitTitle, self).__init__(
+        super().__init__(
             label=_("Split _Version out of Title"), use_underline=True)
         self.set_image(Gtk.Image.new_from_icon_name(
             Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
-        spls = config.gettext("editing", "split_on").split()
-        self.set_sensitive(bool(split_title(value, spls)[1]))
+        tag_spls = config.gettext("editing", "split_on").split()
+        sub_spls = config.gettext("editing", "sub_split_on").split()
+
+        title, versions = split_title(value, tag_spls, sub_spls)
+        if versions:
+            title = title if len(title) <= 64 else title[:64] + "…"
+            versions = [ver if len(ver) <= 64 else ver[:64] + "…"
+                        for ver in versions]
+            string = (", ".join(["{}={}".format(tag, title)] +
+                                ["{}={}".format(self.needs[0], ver) for ver in
+                                 versions]))
+            self.set_label(string)
+
+        self.set_sensitive(bool(versions))
 
     def activated(self, tag, value):
-        spls = config.gettext("editing", "split_on").split()
-        title, versions = split_title(value, spls)
+        tag_spls = config.gettext("editing", "split_on").split()
+        sub_spls = config.gettext("editing", "sub_split_on").split()
+        title, versions = split_title(value, tag_spls, sub_spls)
         return [(tag, title)] + [("version", v) for v in versions]
 
 
-class SplitPerson(Gtk.ImageMenuItem):
+class SplitPerson(TagSplitter):
     tags = ["artist"]
     _order = 0.5
 
     def __init__(self, tag, value):
-        super(SplitPerson, self).__init__(label=self.title, use_underline=True)
+        super().__init__(label=self.title, use_underline=True)
         self.set_image(Gtk.Image.new_from_icon_name(
             Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
-        spls = config.gettext("editing", "split_on").split()
-        self.set_sensitive(bool(split_people(value, spls)[1]))
+        tag_spls = config.gettext("editing", "split_on").split()
+        sub_spls = config.gettext("editing", "sub_split_on").split()
+
+        artist, others = split_people(value, tag_spls, sub_spls)
+        if others:
+            artist = artist if len(artist) <= 64 else artist[:64] + "…"
+            others = [other if len(other) <= 64 else other[:64] + "…"
+                      for other in others]
+            string = (", ".join(["{}={}".format(tag, artist)] +
+                                ["{}={}".format(self.needs[0], o) for o in
+                                 others]))
+            self.set_label(string)
+
+        self.set_sensitive(bool(others))
 
     def activated(self, tag, value):
-        spls = config.gettext("editing", "split_on").split()
-        artist, others = split_people(value, spls)
+        tag_spls = config.gettext("editing", "split_on").split()
+        sub_spls = config.gettext("editing", "sub_split_on").split()
+        artist, others = split_people(value, tag_spls, sub_spls)
         return [(tag, artist)] + [(self.needs[0], o) for o in others]
 
 
@@ -280,7 +322,7 @@ class SplitOriginalArtistFromTitle(SplitPerson):
 class AddTagDialog(Dialog):
 
     def __init__(self, parent, can_change, library):
-        super(AddTagDialog, self).__init__(
+        super().__init__(
             title=_("Add a Tag"), transient_for=qltk.get_top_parent(parent),
             use_header_bar=True)
         self.set_border_width(6)
@@ -368,7 +410,7 @@ class AddTagDialog(Dialog):
         self.show()
         self.__val.set_activates_default(True)
         self.__tag.grab_focus()
-        return super(AddTagDialog, self).run()
+        return super().run()
 
 
 class EditTagsPluginHandler(EditingPluginHandler):
@@ -376,7 +418,7 @@ class EditTagsPluginHandler(EditingPluginHandler):
     Kind = EditTagsPlugin
 
 
-class ListEntry(object):
+class ListEntry:
     """Holds a Comment and some state for the editing process"""
 
     tag = None
@@ -396,14 +438,20 @@ class ListEntry(object):
 class EditTags(Gtk.VBox):
     handler = EditTagsPluginHandler()
 
+    _SPLITTERS: Sequence[Type[TagSplitter]] = sorted(
+        [SplitDisc, SplitTitle, SplitPerformer, SplitArranger, SplitValues,
+         SplitPerformerFromTitle, SplitOriginalArtistFromTitle],
+        key=lambda item: (item._order, item.__name__))
+
     @classmethod
     def init_plugins(cls):
         PluginManager.instance.register_handler(cls.handler)
 
     def __init__(self, parent, library):
-        super(EditTags, self).__init__(spacing=12)
+        super().__init__(spacing=12)
         self.title = _("Edit Tags")
         self.set_border_width(12)
+        self._group_info = None
 
         model = ObjectStore()
         view = RCMHintedTreeView(model=model)
@@ -425,6 +473,7 @@ class EditTags(Gtk.VBox):
                     rend.set_property('icon-name', Icons.EDIT)
             else:
                 rend.set_property('icon-name', Icons.CHANGES_PREVENT)
+
         column.set_cell_data_func(render, cdf_write)
         view.append_column(column)
 
@@ -532,7 +581,7 @@ class EditTags(Gtk.VBox):
             model.connect(sig, self.__enable_save, [save, revert])
             connect_obj(model, sig, parent.set_pending, save)
 
-        view.connect('popup-menu', self.__popup_menu, parent)
+        view.connect('popup-menu', self._popup_menu, parent)
         view.connect('button-press-event', self.__button_press)
         view.connect('key-press-event', self.__view_key_press_event)
         selection.connect('changed', self.__tag_select, remove)
@@ -554,6 +603,9 @@ class EditTags(Gtk.VBox):
             # Issue 697: allow Ctrl-s to save.
             self._save.emit('clicked')
             return Gdk.EVENT_STOP
+        elif qltk.is_accel(event, "<Primary>c"):
+            self.__copy_tag_value(event, view)
+            return Gdk.EVENT_STOP
         return Gdk.EVENT_PROPAGATE
 
     def __enable_save(self, *args):
@@ -591,18 +643,25 @@ class EditTags(Gtk.VBox):
             entry.edited = entry.deleted = True
             model.path_changed(path)
 
-    def __popup_menu(self, view, parent):
+    def __item_for(self, view: BaseView,
+                   Item: Type[EditTagsPlugin],
+                   tag: str,
+                   text: str) -> Optional[EditTagsPlugin]:
+        try:
+            item = Item(tag, text)
+        except Exception as e:
+            print_e(f"Couldn't create menu item from {Item} ({e})")
+            return None
+        else:
+            item.connect('activate', self.__menu_activate, view)
+            return item
+
+    def _popup_menu(self, view: BaseView, _parent):
         menu = Gtk.Menu()
 
         view.ensure_popup_selection()
         model, rows = view.get_selection().get_selected_rows()
-        can_change = min([model[path][0].canedit for path in rows])
-
-        items = [SplitDisc, SplitTitle, SplitPerformer, SplitArranger,
-                 SplitValues, SplitPerformerFromTitle,
-                 SplitOriginalArtistFromTitle]
-        items.extend(self.handler.plugins)
-        items.sort(key=lambda item: (item._order, item.__name__))
+        can_change = all(model[path][0].canedit for path in rows)
 
         if len(rows) == 1:
             row = model[rows[0]]
@@ -611,38 +670,77 @@ class EditTags(Gtk.VBox):
             comment = entry.value
             text = comment.text
 
-            for Item in items:
+            split_menu = Gtk.Menu()
+
+            for Item in self._SPLITTERS:
                 if Item.tags and entry.tag not in Item.tags:
                     continue
+                item = self.__item_for(view, Item, entry.tag, text)
+                if not item:
+                    continue
+                vals = item.activated(entry.tag, text)
+                changeable = any(not self._group_info.can_change(k)
+                                 for k in item.needs)
+                fixed = changeable or comment.is_special()
+                if fixed:
+                    item.set_sensitive(False)
+                if len(vals) > 1 and vals[1][1]:
+                    split_menu.append(item)
+            if split_menu.get_children():
+                split_menu.append(SeparatorMenuItem())
 
-                try:
-                    b = Item(entry.tag, text)
-                except:
-                    util.print_exc()
+            plugins = self.handler.plugins
+            print_d(f"Adding {len(plugins)} plugin(s) to menu: "
+                    f"{', '.join(p.__name__ for p in plugins)}")
+            for p_cls in plugins:
+                item = self.__item_for(view, p_cls, entry.tag, text)
+                if not item:
+                    continue
+                results = item.activated(entry.tag, text)
+                # Only enable for the user if the plugin would do something
+                item.set_sensitive(results != [(entry.tag, text)])
+                menu.append(item)
+            pref_item = MenuItem(_("_Configure"), Icons.PREFERENCES_SYSTEM)
+            split_menu.append(pref_item)
+
+            def show_prefs(parent):
+                from quodlibet.qltk.exfalsowindow import ExFalsoWindow
+                if isinstance(app.window, ExFalsoWindow):
+                    from quodlibet.qltk.exfalsowindow import PreferencesWindow
+                    window = PreferencesWindow(parent)
                 else:
-                    b.connect('activate', self.__menu_activate, view)
+                    from quodlibet.qltk.prefs import PreferencesWindow
+                    window = PreferencesWindow(parent, open_page="tagging")
+                window.show()
 
-                    if (not min(listmap(self.__songinfo.can_change, b.needs) +
-                                [1])
-                            or comment.is_special()):
-                        b.set_sensitive(False)
+            connect_obj(pref_item, "activate", show_prefs, self)
 
-                    menu.append(b)
+            split_item = MenuItem(_("_Split Tag"), Icons.EDIT_FIND_REPLACE)
 
-            if menu.get_children():
-                menu.append(SeparatorMenuItem())
+            if split_menu.get_children():
+                split_item.set_submenu(split_menu)
+            else:
+                split_item.set_sensitive(False)
 
-        b = MenuItem(_("_Remove"), Icons.LIST_REMOVE)
-        b.connect('activate', self.__remove_tag, view)
-        qltk.add_fake_accel(b, "Delete")
-        menu.append(b)
+            menu.append(split_item)
+
+        copy_b = MenuItem(_("_Copy Value(s)"), Icons.EDIT_COPY)
+        copy_b.connect('activate', self.__copy_tag_value, view)
+        qltk.add_fake_accel(copy_b, "<Primary>c")
+        menu.append(copy_b)
+
+        remove_b = MenuItem(_("_Remove"), Icons.LIST_REMOVE)
+        remove_b.connect('activate', self.__remove_tag, view)
+        qltk.add_fake_accel(remove_b, "Delete")
+        menu.append(remove_b)
 
         menu.show_all()
         # Setting the menu itself to be insensitive causes it to not
         # be dismissed; see #473.
         for c in menu.get_children():
             c.set_sensitive(can_change and c.get_property('sensitive'))
-        b.set_sensitive(True)
+        copy_b.set_sensitive(True)
+        remove_b.set_sensitive(True)
         menu.connect('selection-done', lambda m: m.destroy())
 
         # XXX: Keep reference
@@ -654,9 +752,9 @@ class EditTags(Gtk.VBox):
         remove.set_sensitive(bool(rows))
 
     def __add_new_tag(self, model, tag, value):
-        assert isinstance(value, text_type)
+        assert isinstance(value, str)
         iters = [i for (i, v) in model.iterrows() if v.tag == tag]
-        if iters and not self.__songinfo.can_multiple_values(tag):
+        if iters and not self._group_info.can_multiple_values(tag):
             title = _("Unable to add tag")
             msg = _("Unable to add <b>%s</b>") % util.escape(tag)
             msg += "\n\n"
@@ -675,7 +773,7 @@ class EditTags(Gtk.VBox):
             model.append(row=[entry])
 
     def __add_tag(self, activator, model, library):
-        add = AddTagDialog(self, self.__songinfo.can_change(), library)
+        add = AddTagDialog(self, self._group_info.can_change(), library)
 
         while True:
             resp = add.run()
@@ -683,10 +781,10 @@ class EditTags(Gtk.VBox):
                 break
             tag = add.get_tag()
             value = add.get_value()
-            assert isinstance(value, text_type)
+            assert isinstance(value, str)
             value = massagers.validate(tag, value)
-            assert isinstance(value, text_type)
-            if not self.__songinfo.can_change(tag):
+            assert isinstance(value, str)
+            if not self._group_info.can_change(tag):
                 title = _("Invalid tag")
                 msg = _("Invalid tag <b>%s</b>\n\nThe files currently"
                         " selected do not support editing this tag."
@@ -711,13 +809,26 @@ class EditTags(Gtk.VBox):
             else:
                 model.remove(row.iter)
 
+    def __copy_tag_value(self, activator, view):
+        model, paths = view.get_selection().get_selected_rows()
+        rows = [model[path] for path in paths]
+        values = []
+        for row in rows:
+            entry_text = row[0].value.get_shared_text()
+            if entry_text:
+                values.append(entry_text)
+        text = '\n'.join(values)
+        if len(text) > 0:
+            clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+            clipboard.set_text(text, -1)
+
     def __save_files(self, save, revert, model, library):
         updated = {}
         deleted = {}
         added = {}
         renamed = {}
 
-        for entry in itervalues(model):
+        for entry in model.values():
             if entry.edited and not (entry.deleted or entry.renamed):
                 if entry.origvalue is not None:
                     l = updated.setdefault(entry.tag, [])
@@ -736,7 +847,7 @@ class EditTags(Gtk.VBox):
                 l.append((entry.origtag, entry.value, entry.origvalue))
 
         was_changed = set()
-        songs = self.__songinfo.songs
+        songs = self._group_info.songs
         win = WritingWindow(self, len(songs))
         win.show()
         all_done = False
@@ -750,7 +861,7 @@ class EditTags(Gtk.VBox):
                     break
 
             changed = False
-            for key, values in iteritems(updated):
+            for key, values in updated.items():
                 for (new_value, old_value) in values:
                     if song.can_change(key):
                         if old_value is None:
@@ -759,13 +870,13 @@ class EditTags(Gtk.VBox):
                             song.change(key, old_value.text, new_value.text)
                         changed = True
 
-            for key, values in iteritems(added):
+            for key, values in added.items():
                 for value in values:
                     if song.can_change(key):
                         song.add(key, value.text)
                         changed = True
 
-            for key, values in iteritems(deleted):
+            for key, values in deleted.items():
                 for value in values:
                     if not value.shared:
                         # In case it isn't shared we don't know the actual
@@ -779,7 +890,7 @@ class EditTags(Gtk.VBox):
                         changed = True
 
             save_rename = []
-            for new_tag, values in iteritems(renamed):
+            for new_tag, values in renamed.items():
                 for old_tag, new_value, old_value in values:
                     if (song.can_change(new_tag) and old_tag in song):
                         if not new_value.is_special():
@@ -853,7 +964,7 @@ class EditTags(Gtk.VBox):
         entry = model[path][0]
         if new_tag == entry.tag:
             return
-        elif not self.__songinfo.can_change(new_tag):
+        elif not self._group_info.can_change(new_tag):
             # Can't add the new tag.
             title = _("Invalid tag")
             msg = _("Invalid tag <b>%s</b>\n\nThe files currently"
@@ -923,12 +1034,12 @@ class EditTags(Gtk.VBox):
 
     def _update(self, songs=None):
         if songs is None:
-            songs = self.__songinfo.songs
+            songs = self._group_info.songs
         else:
-            self.__songinfo = AudioFileGroup(songs)
-        songinfo = self.__songinfo
+            self._group_info = AudioFileGroup(songs)
+        info = self._group_info
 
-        keys = listkeys(songinfo)
+        keys = list(info.keys())
         default_tags = get_default_tags()
         keys = set(keys + default_tags)
 
@@ -949,16 +1060,16 @@ class EditTags(Gtk.VBox):
             model.clear()
 
             for tag in sorted(keys, key=custom_sort):
-                canedit = songinfo.can_change(tag)
+                canedit = info.can_change(tag)
 
                 # default tags
-                if tag not in songinfo:
+                if tag not in info:
                     entry = ListEntry(tag, Comment(u""))
                     entry.canedit = canedit
                     model.append(row=[entry])
                     continue
 
-                for value in songinfo[tag]:
+                for value in info[tag]:
                     entry = ListEntry(tag, value)
                     entry.origvalue = value
                     entry.edited = False
@@ -968,7 +1079,7 @@ class EditTags(Gtk.VBox):
                     entry.origtag = ""
                     model.append(row=[entry])
 
-        self._buttonbox.set_sensitive(bool(songinfo.can_change()))
+        self._buttonbox.set_sensitive(bool(info.can_change()))
         self._revert.set_sensitive(False)
         self._remove.set_sensitive(False)
         self._save.set_sensitive(False)
@@ -993,7 +1104,7 @@ class EditTags(Gtk.VBox):
 
     def __tag_editing_started(self, render, editable, path, model, library):
         if not editable.get_completion():
-            tags = self.__songinfo.can_change()
+            tags = self._group_info.can_change()
             if tags is True:
                 tags = USER_TAGS
             completion = qltk.EntryCompletion(tags)
