@@ -1,33 +1,34 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2009 Joe Wreschnig, Michael Urman, Iñigo Serna,
 #                     Steven Robertson
-#           2011-2013 Nick Boultbee
+#           2011-2017 Nick Boultbee
 #           2013      Christoph Reiter
 #           2014      Jan Path
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from gi.repository import Gtk
 
 from quodlibet import config
-from quodlibet import const
 from quodlibet import qltk
 from quodlibet import util
 from quodlibet import app
-from quodlibet.config import RATINGS
+from quodlibet import C_, _
+from quodlibet.config import RATINGS, DurationFormat, DURATION
 
 from quodlibet.qltk.ccb import ConfigCheckButton as CCB
-from quodlibet.qltk.data_editors import MultiStringEditor
-from quodlibet.qltk.entry import ValidatingEntry, UndoEntry, QueryValidator
+from quodlibet.qltk.data_editors import TagListEditor
+from quodlibet.qltk.entry import ValidatingEntry, UndoEntry
+from quodlibet.query._query import Query
 from quodlibet.qltk.scanbox import ScanBox
 from quodlibet.qltk.maskedbox import MaskedBox
 from quodlibet.qltk.songlist import SongList, get_columns
 from quodlibet.qltk.window import UniqueWindow
 from quodlibet.qltk.x import Button, Align
-from quodlibet.qltk import icons
-from quodlibet.util import copool
+from quodlibet.qltk import Icons
+from quodlibet.util import copool, format_time_preferred
 from quodlibet.util.dprint import print_d
 from quodlibet.util.library import emit_signal, get_scan_dirs, scan_library
 from quodlibet.util import connect_obj
@@ -58,110 +59,126 @@ class PreferencesWindow(UniqueWindow):
             ("~#filesize", util.tag("~#filesize"))]
 
         def __init__(self):
-            super(PreferencesWindow.SongList, self).__init__(spacing=12)
+            def create_behaviour_frame():
+                vbox = Gtk.VBox(spacing=6)
+                jump_button = CCB(_("_Jump to playing song automatically"),
+                        'settings', 'jump', populate=True,
+                        tooltip=_("When the playing song changes, "
+                                  "scroll to it in the song list"))
+                autosort_button = CCB(_("Sort songs when tags are modified"),
+                        'song_list', 'auto_sort', populate=True,
+                        tooltip=_("Automatically re-sort songs in the song list when "
+                                  "tags are modified"))
+                vbox.pack_start(jump_button, False, True, 0)
+                vbox.pack_start(autosort_button, False, True, 0)
+                return qltk.Frame(_("Behavior"), child=vbox)
+
+            def create_visible_columns_frame():
+                buttons = {}
+                vbox = Gtk.VBox(spacing=12)
+                table = Gtk.Table.new(3, 3, True)
+                for i, (k, t) in enumerate(self.PREDEFINED_TAGS):
+                    x, y = i % 3, i / 3
+                    buttons[k] = Gtk.CheckButton(label=t, use_underline=True)
+                    table.attach(buttons[k], x, x + 1, y, y + 1)
+                vbox.pack_start(table, False, True, 0)
+                # Other columns
+                hbox = Gtk.HBox(spacing=6)
+                l = Gtk.Label(label=_("_Others:"), use_underline=True)
+                hbox.pack_start(l, False, True, 0)
+                self.others = others = UndoEntry()
+                others.set_sensitive(False)
+                # Stock edit doesn't have ellipsis chars.
+                edit_button = Gtk.Button(
+                    label=_(u"_Edit…"), use_underline=True)
+                edit_button.connect("clicked", self.__config_cols, buttons)
+                edit_button.set_tooltip_text(
+                    _("Add or remove additional column "
+                      "headers"))
+                l.set_mnemonic_widget(edit_button)
+                l.set_use_underline(True)
+                hbox.pack_start(others, True, True, 0)
+                vbox.pack_start(hbox, False, True, 0)
+                b = Gtk.HButtonBox()
+                b.set_layout(Gtk.ButtonBoxStyle.END)
+                b.pack_start(edit_button, True, True, 0)
+                vbox.pack_start(b, True, True, 0)
+                return qltk.Frame(_("Visible Columns"), child=vbox), buttons
+
+            def create_columns_prefs_frame():
+                tiv = Gtk.CheckButton(label=_("Title includes _version"),
+                                      use_underline=True)
+                aio = Gtk.CheckButton(label=_("Artist includes all _people"),
+                                      use_underline=True)
+                aip = Gtk.CheckButton(label=_("Album includes _disc subtitle"),
+                                      use_underline=True)
+                fip = Gtk.CheckButton(label=_("Filename includes _folder"),
+                                      use_underline=True)
+                self._toggle_data = [
+                    (tiv, "title", "~title~version"),
+                    (aip, "album", "~album~discsubtitle"),
+                    (fip, "~basename", "~filename"),
+                    (aio, "artist", "~people")
+                ]
+                t = Gtk.Table.new(2, 2, True)
+                t.attach(tiv, 0, 1, 0, 1)
+                t.attach(aip, 0, 1, 1, 2)
+                t.attach(aio, 1, 2, 0, 1)
+                t.attach(fip, 1, 2, 1, 2)
+                return qltk.Frame(_("Column Preferences"), child=t)
+
+            def create_apply_button():
+                vbox = Gtk.VBox(spacing=12)
+                apply = Button(_("_Apply"))
+                apply.set_tooltip_text(
+                    _("Apply current configuration to song list, "
+                      "adding new columns to the end"))
+                apply.connect('clicked', self.__apply, buttons)
+                # Apply on destroy, else config gets mangled
+                self.connect('destroy', self.__apply, buttons)
+                b = Gtk.HButtonBox()
+                b.set_layout(Gtk.ButtonBoxStyle.END)
+                b.pack_start(apply, True, True, 0)
+                vbox.pack_start(b, True, True, 0)
+                return vbox
+
+            super().__init__(spacing=12)
             self.set_border_width(12)
             self.title = _("Song List")
-
-            # Behaviour
-            vbox = Gtk.VBox(spacing=6)
-            c = CCB(_("_Jump to playing song automatically"),
-                    'settings', 'jump', populate=True,
-                    tooltip=_("When the playing song changes, "
-                              "scroll to it in the song list"))
-            vbox.pack_start(c, False, True, 0)
-            frame = qltk.Frame(_("Behavior"), child=vbox)
-            self.pack_start(frame, False, True, 0)
-
-            # Columns
-            vbox = Gtk.VBox(spacing=12)
-            buttons = {}
-            table = Gtk.Table.new(3, 3, True)
-            cols = get_columns()
-
-            for i, (k, t) in enumerate(self.PREDEFINED_TAGS):
-                x, y = i % 3, i / 3
-                buttons[k] = Gtk.CheckButton(label=t, use_underline=True)
-                if k in cols:
-                    buttons[k].set_active(True)
-                    cols.remove(k)
-                table.attach(buttons[k], x, x + 1, y, y + 1)
-            vbox.pack_start(table, False, True, 0)
-            if "~current" in cols:
-                cols.remove("~current")
-            self.other_cols = cols
-
-            # Other columns
-            hbox = Gtk.HBox(spacing=6)
-            l = Gtk.Label(label=_("_Others:"), use_underline=True)
-            hbox.pack_start(l, False, True, 0)
-            self.others = others = UndoEntry()
-            others.set_sensitive(False)
-            # Stock edit doesn't have ellipsis chars.
-            edit_button = Gtk.Button(
-                label=_(u"_Edit…"), use_underline=True)
-            edit_button.connect("clicked", self.__config_cols)
-            edit_button.set_tooltip_text(_("Add or remove additional column "
-                                           "headers"))
-            l.set_mnemonic_widget(edit_button)
-            l.set_use_underline(True)
-            hbox.pack_start(others, True, True, 0)
-            hbox.pack_start(edit_button, False, True, 0)
-            vbox.pack_start(hbox, False, True, 0)
-
-            frame = qltk.Frame(_("Visible Columns"), child=vbox)
-            self.pack_start(frame, False, True, 0)
-
-            # Column preferences
-            tiv = Gtk.CheckButton(label=_("Title includes _version"),
-                                  use_underline=True)
-            aio = Gtk.CheckButton(label=_("Artist includes all _people"),
-                                  use_underline=True)
-            aip = Gtk.CheckButton(label=_("Album includes _disc subtitle"),
-                                  use_underline=True)
-            fip = Gtk.CheckButton(label=_("Filename includes _folder"),
-                                  use_underline=True)
-            self._toggle_data = [
-                (tiv, "title", "~title~version"),
-                (aip, "album", "~album~discsubtitle"),
-                (fip, "~basename", "~filename"),
-                (aio, "artist", "~people")
-            ]
-            # Turn on the toggles if the toggled version is detected in config
-            for (check, off, on) in self._toggle_data:
-                if on in cols:
-                    buttons[off].set_active(True)
-                    check.set_active(True)
-                    cols.remove(on)
-
-            # Update text once to exclude ticked columns, munged or not
-            others.set_text(", ".join(cols))
-            t = Gtk.Table.new(2, 2, True)
-            t.attach(tiv, 0, 1, 0, 1)
-            t.attach(aip, 0, 1, 1, 2)
-            t.attach(aio, 1, 2, 0, 1)
-            t.attach(fip, 1, 2, 1, 2)
-            frame = qltk.Frame(_("Column Preferences"), child=t)
-            self.pack_start(frame, False, True, 0)
-
-            # Apply button
-            vbox = Gtk.VBox(spacing=12)
-            apply = Gtk.Button(stock=Gtk.STOCK_APPLY)
-            apply.set_tooltip_text(_("Apply current configuration to song "
-                                     "list, adding new columns to the end"))
-            b = Gtk.HButtonBox()
-            b.set_layout(Gtk.ButtonBoxStyle.END)
-            b.pack_start(apply, True, True, 0)
-            vbox.pack_start(b, True, True, 0)
-            self.pack_start(vbox, True, True, 0)
-            apply.connect('clicked', self.__apply, buttons, tiv, aip, fip,
-                          aio)
-            # Apply on destroy, else config gets mangled
-            self.connect('destroy', self.__apply, buttons, tiv, aip, fip, aio)
+            self.pack_start(create_behaviour_frame(), False, True, 0)
+            columns_frame, buttons = create_visible_columns_frame()
+            self.pack_start(columns_frame, False, True, 0)
+            self.pack_start(create_columns_prefs_frame(), False, True, 0)
+            self.pack_start(create_apply_button(), True, True, 0)
+            self.__update(buttons, self._toggle_data, get_columns())
 
             for child in self.get_children():
                 child.show_all()
 
-        def __apply(self, button, buttons, tiv, aip, fip, aio):
+        def __update(self, buttons, toggle_data, columns):
+            """Updates all widgets based on the passed column list"""
+
+            columns = list(columns)
+
+            for key, widget in buttons.items():
+                widget.set_active(key in columns)
+                if key in columns:
+                    columns.remove(key)
+
+            for (check, off, on) in toggle_data:
+                if on in columns:
+                    buttons[off].set_active(True)
+                    check.set_active(True)
+                    columns.remove(on)
+
+            self.others.set_text(", ".join(columns))
+            self.other_cols = columns
+
+        def __get_current_columns(self, buttons):
+            """Given the current column list and the widgets states compute
+            a new column list.
+            """
+
             new_headers = set()
             # Get the checked headers
             for key, name in self.PREDEFINED_TAGS:
@@ -195,14 +212,20 @@ class PreferencesWindow(UniqueWindow):
                     except ValueError:
                         pass
 
+            return result
+
+        def __apply(self, button, buttons):
+            result = self.__get_current_columns(buttons)
             SongList.set_all_column_headers(result)
 
-        def __config_cols(self, button):
+        def __config_cols(self, button, buttons):
             def __closed(widget):
-                self.other_cols = widget.get_strings()
-                self.others.set_text(", ".join(self.other_cols))
+                cols = widget.get_strings()
+                self.__update(buttons, self._toggle_data, cols)
 
-            m = MultiStringEditor(_("Extra Columns"), self.other_cols)
+            columns = self.__get_current_columns(buttons)
+            m = TagListEditor(_("Edit Columns"), columns)
+            m.set_transient_for(qltk.get_top_parent(self))
             m.connect('destroy', __closed)
             m.show()
 
@@ -210,27 +233,65 @@ class PreferencesWindow(UniqueWindow):
         name = "browser"
 
         def __init__(self):
-            super(PreferencesWindow.Browsers, self).__init__(spacing=12)
+            def create_display_frame():
+                vbox = Gtk.VBox(spacing=6)
+                model = Gtk.ListStore(str, str)
+
+                def on_changed(combo):
+                    it = combo.get_active_iter()
+                    if it is None:
+                        return
+                    DURATION.format = model[it][0]
+                    app.window.songlist.info.refresh()
+                    app.window.qexpander.refresh()
+                    # TODO: refresh info windows ideally too (but see #2019)
+
+                def draw_duration(column, cell, model, it, data):
+                    df, example = model[it]
+                    cell.set_property('text', example)
+
+                for df in sorted(DurationFormat.values):
+                    # 4954s == longest ever CD, FWIW
+                    model.append([df, format_time_preferred(4954, df)])
+                duration = Gtk.ComboBox(model=model)
+                cell = Gtk.CellRendererText()
+                duration.pack_start(cell, True)
+                duration.set_cell_data_func(cell, draw_duration, None)
+                index = sorted(DurationFormat.values).index(DURATION.format)
+                duration.set_active(index)
+                duration.connect('changed', on_changed)
+                hbox = Gtk.HBox(spacing=6)
+                label = Gtk.Label(label=_("Duration totals") + ":",
+                                  use_underline=True)
+                label.set_mnemonic_widget(duration)
+                hbox.pack_start(label, False, True, 0)
+                hbox.pack_start(duration, False, True, 0)
+
+                vbox.pack_start(hbox, False, True, 0)
+                return qltk.Frame(_("Display"), child=vbox)
+
+            def create_search_frame():
+                vb = Gtk.VBox(spacing=6)
+                hb = Gtk.HBox(spacing=6)
+                l = Gtk.Label(label=_("_Global filter:"))
+                l.set_use_underline(True)
+                e = ValidatingEntry(Query.validator)
+                e.set_text(config.get("browsers", "background"))
+                e.connect('changed', self._entry, 'background', 'browsers')
+                e.set_tooltip_text(
+                    _("Apply this query in addition to all others"))
+                l.set_mnemonic_widget(e)
+                hb.pack_start(l, False, True, 0)
+                hb.pack_start(e, True, True, 0)
+                vb.pack_start(hb, False, True, 0)
+                # Translators: The heading of the preference group, no action
+                return qltk.Frame(C_("heading", "Search"), child=vb)
+
+            super().__init__(spacing=12)
             self.set_border_width(12)
             self.title = _("Browsers")
-
-            # Search
-            vb = Gtk.VBox(spacing=6)
-            hb = Gtk.HBox(spacing=6)
-            l = Gtk.Label(label=_("_Global filter:"))
-            l.set_use_underline(True)
-            e = ValidatingEntry(QueryValidator)
-            e.set_text(config.get("browsers", "background"))
-            e.connect('changed', self._entry, 'background', 'browsers')
-            e.set_tooltip_text(_("Apply this query in addition to all others"))
-            l.set_mnemonic_widget(e)
-            hb.pack_start(l, False, True, 0)
-            hb.pack_start(e, True, True, 0)
-            vb.pack_start(hb, False, True, 0)
-
-            # Translators: The heading of the preference group, no action
-            f = qltk.Frame(C_("heading", "Search"), child=vb)
-            self.pack_start(f, False, True, 0)
+            self.pack_start(create_search_frame(), False, True, 0)
+            self.pack_start(create_display_frame(), False, True, 0)
 
             # Ratings
             vb = Gtk.VBox(spacing=6)
@@ -250,14 +311,7 @@ class PreferencesWindow(UniqueWindow):
             f = qltk.Frame(_("Ratings"), child=vbox)
             self.pack_start(f, False, True, 0)
 
-            # Album Art
             vb = Gtk.VBox(spacing=6)
-            c = CCB(_("_Use rounded corners on thumbnails"),
-                    'albumart', 'round', populate=True,
-                    tooltip=_("Round the corners of album artwork thumbnail "
-                              "images."))
-            c.connect('toggled', self.__toggle_round_corners)
-            vb.pack_start(c, False, True, 0)
 
             # Filename choice algorithm config
             cb = CCB(_("Prefer _embedded art"),
@@ -267,15 +321,19 @@ class PreferencesWindow(UniqueWindow):
             vb.pack_start(cb, False, True, 0)
 
             hb = Gtk.HBox(spacing=3)
-            cb = CCB(_("_Fixed image filename:"),
+
+            preferred_image_filename_tooltip = _(
+                "The album art image file(s) to use when available "
+                "(supports wildcards). If you want to supply more "
+                "than one, separate them with commas.")
+
+            cb = CCB(_("_Preferred image filename(s):"),
                      'albumart', 'force_filename', populate=True,
-                     tooltip=_("The single image filename to use if "
-                               "selected"))
+                     tooltip=preferred_image_filename_tooltip)
             hb.pack_start(cb, False, True, 0)
 
             entry = UndoEntry()
-            entry.set_tooltip_text(
-                    _("The album art image file to use when forced"))
+            entry.set_tooltip_text(preferred_image_filename_tooltip)
             entry.set_text(config.get("albumart", "filename"))
             entry.connect('changed', self.__changed_text, 'filename')
             # Disable entry when not forcing
@@ -293,9 +351,6 @@ class PreferencesWindow(UniqueWindow):
         def __changed_text(self, entry, name):
             config.set('albumart', name, entry.get_text())
 
-        def __toggle_round_corners(self, *args):
-            qltk.redraw_all_toplevels()
-
         def __toggled_force_filename(self, cb, fn_entry):
             fn_entry.set_sensitive(cb.get_active())
 
@@ -306,7 +361,7 @@ class PreferencesWindow(UniqueWindow):
         name = "playback"
 
         def __init__(self):
-            super(PreferencesWindow.Player, self).__init__(spacing=12)
+            super().__init__(spacing=12)
             self.set_border_width(12)
             self.title = _("Playback")
 
@@ -332,7 +387,7 @@ class PreferencesWindow(UniqueWindow):
             fb_label.set_mnemonic_widget(fb_spin)
 
             pre_amp_gain = config.getfloat("player", "pre_amp_gain", 0.0)
-            adj = Gtk.Adjustment.new(pre_amp_gain, -6, 6, 0.5, 0.5, 0.0)
+            adj = Gtk.Adjustment.new(pre_amp_gain, -12, 12, 0.5, 0.5, 0.0)
             adj.connect('value-changed', self.__changed,
                         'player', 'pre_amp_gain')
             pre_spin = Gtk.SpinButton(adjustment=adj)
@@ -377,18 +432,29 @@ class PreferencesWindow(UniqueWindow):
 
             self.pack_start(f, False, True, 0)
 
+            vbox = Gtk.VBox()
+            c = CCB(_("_Continue playback on startup"),
+                    "player", "restore_playing", populate=True,
+                    tooltip=_("If music is playing on shutdown, automatically "
+                              "start playing on next startup"))
+            vbox.pack_start(c, False, False, 0)
+
+            f = qltk.Frame(_("Startup"), child=vbox)
+            self.pack_start(f, False, True, 0)
+
             for child in self.get_children():
                 child.show_all()
 
         def __toggled_gain(self, activator, widgets):
-            if app.player: # tests
-                app.player.volume = app.player.volume
+            if app.player:
+                # tests
+                app.player.reset_replaygain()
             for widget in widgets:
                 widget.set_sensitive(activator.get_active())
 
         def __changed(self, adj, section, name):
             config.set(section, name, str(adj.get_value()))
-            app.player.volume = app.player.volume
+            app.player.reset_replaygain()
 
     class Tagging(Gtk.VBox):
         name = "tagging"
@@ -506,16 +572,25 @@ class PreferencesWindow(UniqueWindow):
             hb.pack_start(bayes_label, False, True, 0)
             hb.pack_start(bayes_spin, False, True, 0)
             vb.pack_start(hb, True, True, 0)
-            cb = CCB(_("Save ratings and play _counts"),
+            cb = CCB(_("Save ratings and play _counts in tags"),
                      "editing", "save_to_songs", populate=True)
+
+            def update_entry(widget, email_entry):
+                email_entry.set_sensitive(widget.get_active())
+
             vb.pack_start(cb, True, True, 0)
             hb = Gtk.HBox(spacing=6)
             lab = Gtk.Label(label=_("_Email:"))
             entry = UndoEntry()
-            entry.set_tooltip_text(_("Ratings and play counts will be set "
-                                     "for this email address"))
+            entry.set_tooltip_text(_("Ratings and play counts will be saved "
+                                     "in tags for this email address"))
             entry.set_text(config.get("editing", "save_email"))
             entry.connect('changed', self.__changed, 'editing', 'save_email')
+
+            # Disable the entry if not saving to tags
+            cb.connect('clicked', update_entry, entry)
+            update_entry(cb, entry)
+
             hb.pack_start(lab, False, True, 0)
             hb.pack_start(entry, True, True, 0)
             lab.set_mnemonic_widget(entry)
@@ -532,34 +607,66 @@ class PreferencesWindow(UniqueWindow):
                      tooltip=_("Save changes to tags without confirmation "
                                "when editing multiple files"))
             vbox.pack_start(cb, False, True, 0)
-            hb = Gtk.HBox(spacing=6)
-            e = UndoEntry()
-            e.set_text(config.get("editing", "split_on"))
-            e.connect('changed', self.__changed, 'editing', 'split_on')
-            e.set_tooltip_text(
-                _("A list of separators to use when splitting tag values. "
-                  "The list is space-separated"))
 
-            def do_revert_split(button, section, option):
+            split_entry = UndoEntry()
+            split_entry.set_text(config.get("editing", "split_on"))
+            split_entry.connect('changed', self.__changed, 'editing',
+                                'split_on')
+            split_entry.set_tooltip_text(
+                _("A set of separators to use when splitting tag values "
+                  "in the tag editor. "
+                  "The list is space-separated."))
+            split_entry.props.expand = True
+
+            sub_entry = UndoEntry()
+            sub_entry.set_text(config.get("editing", "sub_split_on"))
+            sub_entry.connect('changed', self.__changed, 'editing',
+                              'sub_split_on')
+            sub_entry.set_tooltip_text(
+                _("A set of separators to use when extracting subtags from "
+                  "tags in the tag editor. "
+                  "The list is space-separated, and each entry must only "
+                  "contain two characters."))
+            sub_entry.props.expand = True
+
+            def do_revert_split(button, entry, section, option):
                 config.reset(section, option)
-                e.set_text(config.get(section, option))
+                entry.set_text(config.get(section, option))
+
+            split_revert = Button(_("_Revert"), Icons.DOCUMENT_REVERT)
+            split_revert.connect("clicked", do_revert_split, split_entry,
+                                 "editing", "split_on")
+            split_label = Gtk.Label(label=_("Split _tag on:"))
+            split_label.set_use_underline(True)
+            split_label.set_mnemonic_widget(split_entry)
+
+            sub_revert = Button(_("_Revert"), Icons.DOCUMENT_REVERT)
+            sub_revert.connect("clicked", do_revert_split, sub_entry,
+                               "editing", "sub_split_on")
+            sub_label = Gtk.Label(label=_("Split _subtag on:"))
+            sub_label.set_use_underline(True)
+            sub_label.set_mnemonic_widget(split_entry)
+
+            split_align = Align(halign=Gtk.Align.START)
+            split_align.add(split_label)
+            sub_align = Align(halign=Gtk.Align.START)
+            sub_align.add(sub_label)
+
+            grid = Gtk.Grid(column_spacing=6, row_spacing=6)
+            grid.add(split_align)
+            grid.add(split_entry)
+            grid.add(split_revert)
+            grid.attach(sub_align, 0, 1, 1, 1)
+            grid.attach(sub_entry, 1, 1, 1, 1)
+            grid.attach(sub_revert, 2, 1, 1, 1)
+            grid.props.expand = False
+
+            vbox.pack_start(grid, False, False, 6)
 
-            split_revert = Gtk.Button()
-            split_revert.add(Gtk.Image.new_from_stock(
-                             Gtk.STOCK_REVERT_TO_SAVED, Gtk.IconSize.MENU))
-            split_revert.connect("clicked", do_revert_split, "editing",
-                                 "split_on")
-            l = Gtk.Label(label=_("Split _on:"))
-            l.set_use_underline(True)
-            l.set_mnemonic_widget(e)
-            hb.pack_start(l, False, True, 0)
-            hb.pack_start(e, True, True, 0)
-            hb.pack_start(split_revert, False, True, 0)
-            vbox.pack_start(hb, False, True, 0)
             return vbox
 
         def __init__(self):
-            super(PreferencesWindow.Tagging, self).__init__(spacing=12)
+            super().__init__(spacing=12)
             self.set_border_width(12)
             self.title = _("Tags")
             self._songs = []
@@ -580,7 +687,7 @@ class PreferencesWindow(UniqueWindow):
             config.set(section, name, str(entry.get_value()))
             print_d("Signalling \"changed\" to entire library. Hold tight...")
             # Cache over clicks
-            self._songs = self._songs or app.library.values()
+            self._songs = self._songs or list(app.library.values())
             copool.add(emit_signal, self._songs, funcid="library changed",
                        name=_("Updating for new ratings"))
 
@@ -588,11 +695,11 @@ class PreferencesWindow(UniqueWindow):
         name = "library"
 
         def __init__(self):
-            super(PreferencesWindow.Library, self).__init__(spacing=12)
+            super().__init__(spacing=12)
             self.set_border_width(12)
             self.title = _("Library")
 
-            cb = CCB(_("_Refresh library on start"),
+            cb = CCB(_("Scan library _on start"),
                      "library", "refresh_on_start", populate=True)
             scan_dirs = ScanBox()
 
@@ -602,14 +709,14 @@ class PreferencesWindow(UniqueWindow):
             def refresh_cb(button):
                 scan_library(app.library, force=False)
 
-            refresh = qltk.Button(_("Re_fresh Library"), Gtk.STOCK_REFRESH)
+            refresh = qltk.Button(_("_Scan Library"), Icons.VIEW_REFRESH)
             refresh.connect("clicked", refresh_cb)
             refresh.set_tooltip_text(_("Check for changes in your library"))
 
             def reload_cb(button):
                 scan_library(app.library, force=True)
 
-            reload_ = qltk.Button(_("Re_load Library"), Gtk.STOCK_REFRESH)
+            reload_ = qltk.Button(_("Re_build Library"), Icons.VIEW_REFRESH)
             reload_.connect("clicked", reload_cb)
             reload_.set_tooltip_text(
                 _("Reload all songs in your library. "
@@ -634,31 +741,35 @@ class PreferencesWindow(UniqueWindow):
             for child in self.get_children():
                 child.show_all()
 
-    def __init__(self, parent):
+    def __init__(self, parent, open_page=None):
         if self.is_not_unique():
             return
-        super(PreferencesWindow, self).__init__()
+        super().__init__()
         self.current_scan_dirs = get_scan_dirs()
-        self.set_title(_("Preferences") + " - Quod Libet")
+        self.set_title(_("Preferences"))
         self.set_resizable(False)
         self.set_transient_for(qltk.get_top_parent(parent))
 
         self.__notebook = notebook = qltk.Notebook()
-        for Page in [self.SongList, self.Browsers, self.Player,
-                     self.Library, self.Tagging]:
+        pages = [self.SongList, self.Browsers, self.Player, self.Library,
+                 self.Tagging]
+        for Page in pages:
             page = Page()
             page.show()
             notebook.append_page(page)
 
-        page_name = config.get("memory", "prefs_page", "")
-        self.set_page(page_name)
+        if open_page in [page.name for page in pages]:
+            self.set_page(open_page)
+        else:
+            page_name = config.get("memory", "prefs_page", "")
+            self.set_page(page_name)
 
         def on_switch_page(notebook, page, page_num):
             config.set("memory", "prefs_page", page.name)
 
         notebook.connect("switch-page", on_switch_page)
 
-        close = Button(_("_Close"), icons.WINDOW_CLOSE)
+        close = Button(_("_Close"), Icons.WINDOW_CLOSE)
         connect_obj(close, 'clicked', lambda x: x.destroy(), self)
         button_box = Gtk.HButtonBox()
         button_box.set_layout(Gtk.ButtonBoxStyle.END)
@@ -687,6 +798,7 @@ class PreferencesWindow(UniqueWindow):
                 notebook.set_current_page(p)
 
     def __destroy(self):
-        config.write(const.CONFIG)
+        config.save()
         if self.current_scan_dirs != get_scan_dirs():
+            print_d("Library paths have changed, re-scanning...")
             scan_library(app.library, force=False)
