@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna
 #           2016-2017 Nick Boultbee
 #                2017 Fredrik Strupe
@@ -30,7 +29,7 @@ from quodlibet.qltk.menubutton import SmallMenuButton
 from quodlibet.qltk.songmodel import PlaylistModel
 from quodlibet.qltk.playorder import OrderInOrder, OrderShuffle
 from quodlibet.qltk.x import ScrolledWindow, SymbolicIconImage, \
-    SmallImageButton, MenuItem
+    SmallImageButton, SmallImageToggleButton, MenuItem, RadioMenuItem
 from quodlibet.qltk.songlistcolumns import CurrentColumn
 
 QUEUE = os.path.join(quodlibet.get_user_dir(), "queue")
@@ -40,7 +39,7 @@ class PlaybackStatusIcon(Gtk.Box):
     """A widget showing a play/pause/stop symbolic icon"""
 
     def __init__(self):
-        super(PlaybackStatusIcon, self).__init__()
+        super().__init__()
         self._icons = {}
 
     def _set(self, name):
@@ -83,7 +82,7 @@ class ExpandBoxHack(Gtk.HBox):
 class QueueExpander(Gtk.Expander):
 
     def __init__(self, library, player):
-        super(QueueExpander, self).__init__(spacing=3)
+        super().__init__(spacing=3)
         sw = ScrolledWindow()
         sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
         sw.set_shadow_type(Gtk.ShadowType.IN)
@@ -119,6 +118,31 @@ class QueueExpander(Gtk.Expander):
 
         self.set_label_fill(True)
 
+        mode_menu = Gtk.Menu()
+
+        norm_mode_item = RadioMenuItem(
+            label=_("Ephemeral"),
+            tooltip_text=_("Remove songs from the queue after playing them"),
+            group=None)
+        mode_menu.append(norm_mode_item)
+        norm_mode_item.set_active(True)
+        norm_mode_item.connect("toggled",
+                                lambda b: self.__keep_songs_enable(False))
+
+        keep_mode_item = RadioMenuItem(
+            label=_("Persistent"),
+            tooltip_text=_("Keep songs in the queue after playing them"),
+            group=norm_mode_item)
+        mode_menu.append(keep_mode_item)
+        keep_mode_item.connect("toggled",
+                               lambda b: self.__keep_songs_enable(True))
+        keep_mode_item.set_active(
+                config.getboolean("memory", "queue_keep_songs", False))
+
+        mode_item = MenuItem(_("Mode"), Icons.SYSTEM_RUN)
+        mode_item.set_submenu(mode_menu)
+        menu.append(mode_item)
+
         rand_checkbox = ConfigCheckMenuItem(
                 _("_Random"), "memory", "shufflequeue", populate=True)
         rand_checkbox.connect('toggled', self.__queue_shuffle)
@@ -130,19 +154,6 @@ class QueueExpander(Gtk.Expander):
             populate=True)
         menu.append(stop_checkbox)
 
-        queue_ignore_cb = ConfigCheckMenuItem(
-            _("Ignore"), "memory", "queue_ignore",
-            populate=True)
-        menu.append(queue_ignore_cb)
-
-        keep_checkbox = ConfigCheckMenuItem(
-            _("Keep Songs"), "memory", "queue_keep_songs",
-            populate=True)
-        keep_checkbox.set_tooltip_text(
-            _("Keep songs in the queue after playing them"))
-        keep_checkbox.connect("activate", self.__keep_songs_activated)
-        menu.append(keep_checkbox)
-
         clear_item = MenuItem(_("_Clear Queue"), Icons.EDIT_CLEAR)
         menu.append(clear_item)
         clear_item.connect("activate", self.__clear_queue)
@@ -159,6 +170,19 @@ class QueueExpander(Gtk.Expander):
 
         outer.pack_start(button, False, False, 0)
 
+        toggle = SmallImageToggleButton(
+            image=SymbolicIconImage(Icons.SYSTEM_LOCK_SCREEN,
+                                    Gtk.IconSize.MENU),
+            relief=Gtk.ReliefStyle.NONE,
+            tooltip_text=_(
+                "Disable queue - the queue will be ignored when playing"))
+        disabled = config.getboolean("memory", "queue_disable", False)
+        toggle.props.active = disabled
+        self.__queue_disable(disabled)
+        toggle.connect('toggled',
+                       lambda b: self.__queue_disable(b.props.active))
+        outer.pack_start(toggle, False, False, 6)
+
         close_button = SmallImageButton(
             image=SymbolicIconImage("window-close", Gtk.IconSize.MENU),
             relief=Gtk.ReliefStyle.NONE)
@@ -203,9 +227,6 @@ class QueueExpander(Gtk.Expander):
         connect_destroy(
             player, 'song-ended', self.__update_queue_stop, self.queue.model)
 
-        self._last_queue_song = None
-        self._curr_song_index = None
-
         # to make the children clickable if mapped
         # ....no idea why, but works
         def hack(expander):
@@ -246,9 +267,9 @@ class QueueExpander(Gtk.Expander):
     def __clear_queue(self, activator):
         self.model.clear()
 
-    def __keep_songs_activated(self, activator):
-        keep_song = config.getboolean("memory", "queue_keep_songs", False)
-        if keep_song:
+    def __keep_songs_enable(self, enabled):
+        config.set("memory", "queue_keep_songs", enabled)
+        if enabled:
             self.queue.set_first_column_type(CurrentColumn)
         else:
             for col in self.queue.get_columns():
@@ -289,25 +310,14 @@ class QueueExpander(Gtk.Expander):
     def __update_queue_stop(self, player, song, stopped, model):
         enabled = config.getboolean("memory", "queue_stop_at_end", False)
         songs_left = len(model.get())
-        queue_empty = songs_left == 0 and song is self._last_queue_song
-        queue_finished = (self._curr_song_index and
-                          self._curr_song_index + 1 >= songs_left)
-        if (enabled and not stopped and (queue_empty or queue_finished)):
+        if (enabled
+                and not stopped
+                and songs_left == 0
+                and self._queue_sourced):
             app.player.stop()
-        else:
-            self.queue_finished = False
 
     def __song_started(self, player, song, model):
-        songs_left = len(model.get())
-        if songs_left == 0:
-            self._last_queue_song = None
-        else:
-            self._last_queue_song = song
-
-        if model.current_path:
-            self._curr_song_index = int(model.current_path.to_string())
-        else:
-            self._curr_song_index = None
+        self._queue_sourced = model.sourced
 
     def __expand(self, widget, prop, menu_button):
         expanded = self.get_expanded()
@@ -321,6 +331,10 @@ class QueueExpander(Gtk.Expander):
         menu_button.set_property('visible', expanded)
         config.set("memory", "queue_expanded", str(expanded))
 
+    def __queue_disable(self, disabled):
+        config.set("memory", "queue_disable", disabled)
+        self.queue.set_sensitive(not disabled)
+
 
 class QueueModel(PlaylistModel):
     """Own class for debugging"""
@@ -332,7 +346,7 @@ class PlayQueue(SongList):
     _activated = False
 
     def __init__(self, library, player):
-        super(PlayQueue, self).__init__(library, player, model_cls=QueueModel)
+        super().__init__(library, player, model_cls=QueueModel)
         keep_song = config.getboolean("memory", "queue_keep_songs", False)
         if keep_song:
             self.set_first_column_type(CurrentColumn)
