@@ -1,5 +1,5 @@
-# -*- coding: utf-8 -*-
 # Copyright 2013 Christoph Reiter
+#           2020 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -7,15 +7,17 @@
 # (at your option) any later version.
 
 import re
+from typing import Tuple, Text, List
 
+from quodlibet.formats import TIME_TAGS
 from quodlibet import config
 from quodlibet import util
-from quodlibet.compat import text_type
-
+from quodlibet.formats import AudioFile
 from quodlibet.pattern import XMLFromMarkupPattern as XMLFromPattern
+from quodlibet.util.string.date import format_date
 
 
-class PaneConfig(object):
+class PaneConfig:
     """Row pattern format: 'categorize_pattern:display_pattern'
 
     * display_pattern is optional (fallback: ~#tracks)
@@ -27,14 +29,17 @@ class PaneConfig(object):
     """
 
     def __init__(self, row_pattern):
-        parts = re.split(r"(?<!\\):", row_pattern)
-        parts = list(map(lambda p: p.replace(r"\:", ":"), parts))
+        parts = [p.replace(r"\:", ":")
+                 for p in (re.split(r"(?<!\\):", row_pattern))]
 
         is_numeric = lambda s: s[:2] == "~#" and "~" not in s[2:]
         is_pattern = lambda s: '<' in s
         f_round = lambda s: (isinstance(s, float) and "%.2f" % s) or s
 
-        disp = (len(parts) >= 2 and parts[1]) or r"[i](<~#tracks>)[/i]"
+        def is_date(s):
+            return s in TIME_TAGS
+
+        disp = parts[1] if len(parts) >= 2 else r"[i](<~#tracks>)[/i]"
         cat = parts[0]
 
         if is_pattern(cat):
@@ -50,13 +55,19 @@ class PaneConfig(object):
             title = util.tag(cat)
             tags = util.tagsplit(cat)
             has_markup = False
-            if is_numeric(cat):
-
-                def format(song):
-                    v = text_type(f_round(song(cat)))
+            if is_date(cat):
+                def format(song: AudioFile) -> List[Tuple[Text, Text]]:
+                    fmt = config.gettext("settings",
+                                         "datecolumn_timestamp_format")
+                    date_str = format_date(song(cat), fmt)
+                    return [(date_str, date_str)]
+            elif is_numeric(cat):
+                def format(song: AudioFile) -> List[Tuple[Text, Text]]:
+                    v = str(f_round(song(cat)))
                     return [(v, v)]
             else:
-                format = lambda song: song.list_separate(cat)
+                def format(song: AudioFile) -> List[Tuple[Text, Text]]:
+                    return song.list_separate(cat)
 
         if is_pattern(disp):
             try:
@@ -66,7 +77,7 @@ class PaneConfig(object):
             format_display = pd.format
         else:
             if is_numeric(disp):
-                format_display = lambda coll: text_type(f_round(coll(disp)))
+                format_display = lambda coll: str(f_round(coll(disp)))
             else:
                 format_display = lambda coll: util.escape(coll.comma(disp))
 
