@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 # Copyright 2012-2016 Ryan "ZDBioHazard" Turner <zdbiohazard2@gmail.com>
-#           2016-2017 Nick Boultbee
+#           2016-2020 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,13 +14,13 @@ from quodlibet.order.reorder import Reorder
 from quodlibet.plugins.playorder import ShufflePlugin
 from quodlibet.order import OrderRemembered
 from quodlibet.qltk import Icons
-from quodlibet.compat import iteritems
 
 
 class PlaycountEqualizer(ShufflePlugin, OrderRemembered):
     PLUGIN_ID = "playcounteq"
     PLUGIN_NAME = _("Playcount Equalizer")
-    PLUGIN_DESC = _("Shuffle, preferring songs with fewer total plays.")
+    PLUGIN_DESC = _("Adds a shuffle mode "
+                    "that prefers songs with fewer total plays.")
     PLUGIN_ICON = Icons.MEDIA_PLAYLIST_SHUFFLE
     display_name = _("Prefer less played")
 
@@ -29,7 +28,7 @@ class PlaycountEqualizer(ShufflePlugin, OrderRemembered):
 
     # Select the next track.
     def next(self, playlist, current):
-        super(PlaycountEqualizer, self).next(playlist, current)
+        super().next(playlist, current)
 
         remaining = self.remaining(playlist)
 
@@ -40,11 +39,11 @@ class PlaycountEqualizer(ShufflePlugin, OrderRemembered):
         # Set-up the search information.
         max_count = max([song('~#playcount') for song in remaining.values()])
         weights = {i: max_count - song('~#playcount')
-                   for i, song in iteritems(remaining)}
+                   for i, song in remaining.items()}
         choice = int(max(1, math.ceil(sum(weights) * random.random())))
 
         # Search for a track.
-        for i, weight in iteritems(weights):
+        for i, weight in weights.items():
             choice -= weight
             if choice <= 0:
                 return playlist.get_iter([i])
