@@ -1,11 +1,11 @@
-# -*- coding: utf-8 -*-
 # Copyright 2006 Joe Wreschnig
-#           2013 Nick Boultbee
+#           2011-2021 Nick Boultbee
 #           2013,2014 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 """Base library classes.
 
@@ -16,25 +16,24 @@ least useful but most content-agnostic.
 import os
 import shutil
 import time
+from pathlib import Path
+from typing import Set, Optional, Generator
 
 from gi.repository import GObject
-from senf import fsn2text, fsnative
 
 from quodlibet import _
-from quodlibet.formats import MusicFile, AudioFileError, load_audio_files, \
-    dump_audio_files, SerializationError
-from quodlibet.query import Query
+from quodlibet import formats
+from quodlibet import util
+from quodlibet.formats import (MusicFile, AudioFileError, load_audio_files,
+                               dump_audio_files, SerializationError, AudioFile)
 from quodlibet.qltk.notif import Task
+from quodlibet.query import Query
 from quodlibet.util.atomic import atomic_save
 from quodlibet.util.collection import Album
 from quodlibet.util.collections import DictMixin
-from quodlibet import util
-from quodlibet import formats
 from quodlibet.util.dprint import print_d, print_w
-from quodlibet.util.path import unexpand, mkdir, normalize_path, ishidden, \
-    ismount
-from quodlibet.compat import iteritems, iterkeys, itervalues, listkeys, \
-    listvalues, listfilter
+from quodlibet.util.path import unexpand, mkdir, normalize_path, ishidden, ismount
+from senf import fsn2text, fsnative
 
 
 class Library(GObject.GObject, DictMixin):
@@ -64,7 +63,7 @@ class Library(GObject.GObject, DictMixin):
     dirty = False
 
     def __init__(self, name=None):
-        super(Library, self).__init__()
+        super().__init__()
         self._contents = {}
         self._name = name
         if self.librarian is not None and name is not None:
@@ -90,14 +89,14 @@ class Library(GObject.GObject, DictMixin):
 
         if not items:
             return
-        if self.librarian and self in itervalues(self.librarian.libraries):
-            print_d("Changing %d items via librarian." % len(items), self)
+        if self.librarian and self in self.librarian.libraries.values():
+            print_d(f"Changing {len(items)} items via librarian.", self._name)
             self.librarian.changed(items)
         else:
             items = {item for item in items if item in self}
             if not items:
                 return
-            print_d("Changing %d items directly." % len(items), self)
+            print_d(f"Changing {len(items)} items directly.", self._name)
             self._changed(items)
 
     def _changed(self, items):
@@ -106,22 +105,22 @@ class Library(GObject.GObject, DictMixin):
         # Called by the changed method and Librarians.
         if not items:
             return
-        print_d("Changing %d items." % len(items), self)
+        print_d(f"Changing {len(items)} items.", self._name)
         self.dirty = True
         self.emit('changed', items)
 
     def __iter__(self):
         """Iterate over the items in the library."""
-        return itervalues(self._contents)
+        return iter(self._contents.values())
 
     def iteritems(self):
-        return iteritems(self._contents)
+        return iter(self._contents.items())
 
     def iterkeys(self):
-        return iterkeys(self._contents)
+        return iter(self._contents.keys())
 
     def itervalues(self):
-        return itervalues(self._contents)
+        return iter(self._contents.values())
 
     def __len__(self):
         """The number of items in the library."""
@@ -143,7 +142,7 @@ class Library(GObject.GObject, DictMixin):
            (see FileLibrary with masked items)
         """
 
-        return listvalues(self)
+        return list(self.values())
 
     def keys(self):
         return self._contents.keys()
@@ -155,7 +154,7 @@ class Library(GObject.GObject, DictMixin):
         """Load (add) an item into this library"""
         # Subclasses should override this if they want to check
         # item validity; see `FileLibrary`.
-        print_d("Loading %r." % item.key, self)
+        print_d(f"Loading {item.key!r}", self._name)
         self.dirty = True
         self._contents[item.key] = item
 
@@ -178,7 +177,7 @@ class Library(GObject.GObject, DictMixin):
         if not items:
             return items
 
-        print_d("Adding %d items." % len(items), self)
+        print_d(f"Adding {len(items)} items.", self._name)
         for item in items:
             self._contents[item.key] = item
 
@@ -196,9 +195,9 @@ class Library(GObject.GObject, DictMixin):
         if not items:
             return items
 
-        print_d("Removing %d items." % len(items), self)
+        print_d(f"Removing {len(items)} items.", self._name)
         for item in items:
-            del(self._contents[item.key])
+            del self._contents[item.key]
 
         self.dirty = True
         self.emit('removed', items)
@@ -235,7 +234,7 @@ def _load_items(filename):
     return items
 
 
-class PicklingMixin(object):
+class PicklingMixin:
     """A mixin to provide persistence of a library by pickling to disk"""
 
     filename = None
@@ -255,7 +254,7 @@ class PicklingMixin(object):
         # sure that non-mounted items are masked
         self._load_init(items)
 
-        print_d("Done loading contents of %r." % filename, self)
+        print_d(f"Done loading contents of {filename!r}", self._name)
 
     def save(self, filename=None):
         """Save the library to the given filename, or the default if `None`"""
@@ -263,23 +262,27 @@ class PicklingMixin(object):
         if filename is None:
             filename = self.filename
 
-        print_d("Saving contents to %r." % filename, self)
+        print_d(f"Saving contents to {filename!r}", self._name)
 
         try:
             dirname = os.path.dirname(filename)
             mkdir(dirname)
             with atomic_save(filename, "wb") as fileobj:
                 fileobj.write(dump_audio_files(self.get_content()))
-                # unhandled SerializationError, shouldn't happen -> better
-                # not replace the library file with nothing
+        except SerializationError:
+            # Can happen when we try to pickle while the library is being
+            # modified, like in the periodic 15min save.
+            # Ignore, as it should try again later or on program exit.
+            util.print_exc()
         except EnvironmentError:
-            print_w("Couldn't save library to path: %r" % filename)
+            print_w(f"Couldn't save library to path {filename!r}")
         else:
             self.dirty = False
 
 
 class PicklingLibrary(Library, PicklingMixin):
     """A library that pickles its contents to disk"""
+
     def __init__(self, name=None):
         print_d("Using pickling persistence for library \"%s\"" % name)
         PicklingMixin.__init__(self)
@@ -298,7 +301,7 @@ class AlbumLibrary(Library):
         self.librarian = None
         print_d("Initializing Album Library to watch %r" % library._name)
 
-        super(AlbumLibrary, self).__init__(
+        super().__init__(
             "AlbumLibrary for %s" % library._name)
 
         self._library = library
@@ -308,7 +311,7 @@ class AlbumLibrary(Library):
         self.__added(library, library.values(), signal=False)
 
     def load(self):
-        # deprectated
+        # deprecated
         pass
 
     def destroy(self):
@@ -382,7 +385,7 @@ class AlbumLibrary(Library):
                 changed.add(self._contents[key])
             else:  # key changed.. look for it in each album
                 to_add.append(song)
-                for key, album in iteritems(self._contents):
+                for key, album in self._contents.items():
                     if song in album.songs:
                         album.songs.remove(song)
                         if not album.songs:
@@ -420,23 +423,23 @@ class SongLibrary(PicklingLibrary):
     """
 
     def __init__(self, *args, **kwargs):
-        super(SongLibrary, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     @util.cached_property
     def albums(self):
         return AlbumLibrary(self)
 
     def destroy(self):
-        super(SongLibrary, self).destroy()
+        super().destroy()
         if "albums" in self.__dict__:
             self.albums.destroy()
 
     def tag_values(self, tag):
         """Return a set of all values for the given tag."""
-        return {value for song in itervalues(self)
+        return {value for song in self.values()
                 for value in song.list(tag)}
 
-    def rename(self, song, newname, changed=None):
+    def rename(self, song, new_name, changed: Optional[Set] = None):
         """Rename a song.
 
         This requires a special method because it can change the
@@ -448,12 +451,14 @@ class SongLibrary(PicklingLibrary):
         If the song exists in multiple libraries you cannot use this
         method. Instead, use the librarian.
         """
-        print_d("Renaming %r to %r" % (song.key, newname), self)
-        del(self._contents[song.key])
-        song.rename(newname)
+        if song.key == new_name:
+            print_d(f"Nothing changed for {new_name!r}")
+            return
+        print_d(f"Renaming {song.key!r} to {new_name!r}", self)
+        del self._contents[song.key]
+        song.rename(new_name)
         self._contents[song.key] = song
         if changed is not None:
-            print_d("%s: Delaying changed signal." % (type(self).__name__,))
             changed.add(song)
         else:
             self.changed({song})
@@ -465,7 +470,7 @@ class SongLibrary(PicklingLibrary):
 
         songs = self.values()
         if text != "":
-            songs = listfilter(Query(text, star).search, songs)
+            songs = list(filter(Query(text, star).search, songs))
         return songs
 
 
@@ -521,7 +526,7 @@ class FileLibrary(PicklingLibrary):
     """
 
     def __init__(self, name=None):
-        super(FileLibrary, self).__init__(name)
+        super().__init__(name)
         self._masked = {}
 
     def _load_init(self, items):
@@ -563,12 +568,12 @@ class FileLibrary(PicklingLibrary):
         No signals will be fired.
         Return a tuple of booleans: (changed, removed)
         """
-        print_d("Loading %r." % item.key, self)
+        print_d(f"Loading {item.key!r}", self._name)
         valid = item.valid()
 
         # The item is fine; add it if it's not present.
         if not force and valid:
-            print_d("%r is valid." % item.key, self)
+            print_d(f"{item.key!r} is valid.", self._name)
             self._contents[item.key] = item
             return False, False
         else:
@@ -576,7 +581,7 @@ class FileLibrary(PicklingLibrary):
             # We're going to reload; this could change the key.  So
             # remove the item if it's currently in.
             try:
-                del(self._contents[item.key])
+                del self._contents[item.key]
             except KeyError:
                 present = False
             else:
@@ -586,25 +591,24 @@ class FileLibrary(PicklingLibrary):
                 try:
                     item.reload()
                 except AudioFileError:
-                    print_d("Error reloading %r." % item.key, self)
-                    util.print_exc()
+                    print_w(f"Error reloading {item.key!r}", self._name)
                     return False, True
                 else:
-                    print_d("Reloaded %r." % item.key, self)
+                    print_d(f"Reloaded {item.key!r}.", self._name)
                     self._contents[item.key] = item
                     return True, False
             elif not item.mounted():
                 # We don't know if the item is okay or not, since
                 # it's not not mounted. If the item was present
                 # we need to mark it as removed.
-                print_d("Masking %r." % item.key, self)
+                print_d(f"Masking {item.key!r}", self._name)
                 self._masked.setdefault(item.mountpoint, {})
                 self._masked[item.mountpoint][item.key] = item
                 return False, present
             else:
                 # The item doesn't exist at all anymore. Mark it as
                 # removed if it was present, otherwise nothing.
-                print_d("Ignoring (so removing) %r." % item.key, self)
+                print_d(f"Ignoring (so removing) {item.key!r}.", self._name)
                 return False, present
 
     def reload(self, item, changed=None, removed=None):
@@ -645,7 +649,7 @@ class FileLibrary(PicklingLibrary):
         buttons in the UI.
         """
 
-        print_d("Rebuilding, force is %s." % force, self)
+        print_d(f"Rebuilding, force is {force}", self._name)
 
         task = Task(_("Library"), _("Checking mount points"))
         if cofuncid:
@@ -653,8 +657,8 @@ class FileLibrary(PicklingLibrary):
         for i, (point, items) in task.list(enumerate(self._masked.items())):
             if ismount(point):
                 self._contents.update(items)
-                del(self._masked[point])
-                self.emit('added', listvalues(items))
+                del self._masked[point]
+                self.emit('added', list(items.values()))
                 yield True
 
         task = Task(_("Library"), _("Scanning library"))
@@ -667,16 +671,15 @@ class FileLibrary(PicklingLibrary):
                 # These numbers are pretty empirical. We should yield more
             # often than we emit signals; that way the main loop stays
             # interactive and doesn't get bogged down in updates.
-            if len(changed) > 100:
+            if len(changed) >= 200:
                 self.emit('changed', changed)
                 changed = set()
-            if len(removed) > 100:
+            if len(removed) >= 200:
                 self.emit('removed', removed)
                 removed = set()
-            if len(changed) > 5 or i % 100 == 0:
+            if len(changed) > 20 or i % 200 == 0:
                 yield True
-        print_d("Removing %d, changing %d." % (len(removed), len(changed)),
-                self)
+        print_d(f"Removing {len(removed)}, changing {len(changed)}).", self._name)
         if removed:
             self.emit('removed', removed)
         if changed:
@@ -721,7 +724,7 @@ class FileLibrary(PicklingLibrary):
         # first scan each path for new files
         paths_to_load = []
         for scan_path in paths:
-            print_d("Scanning %r." % scan_path)
+            print_d(f"Scanning {scan_path}", self._name)
             desc = _("Scanning %s") % (fsn2text(unexpand(scan_path)))
             with Task(_("Library"), desc) as task:
                 if cofuncid:
@@ -765,7 +768,7 @@ class FileLibrary(PicklingLibrary):
     def get_content(self):
         """Return visible and masked items"""
 
-        items = listvalues(self)
+        items = list(self.values())
         for masked in self._masked.values():
             items.extend(masked.values())
 
@@ -782,23 +785,23 @@ class FileLibrary(PicklingLibrary):
             point = item.mountpoint
         except AttributeError:
             # Checking a key.
-            for point in itervalues(self._masked):
+            for point in self._masked.values():
                 if item in point:
                     return True
         else:
             # Checking a full item.
-            return item in itervalues(self._masked.get(point, {}))
+            return item in self._masked.get(point, {}).values()
 
     def unmask(self, point):
-        print_d("Unmasking %r." % point, self)
+        print_d(f"Unmasking {point!r}", self._name)
         items = self._masked.pop(point, {})
         if items:
             self.add(items.values())
 
     def mask(self, point):
-        print_d("Masking %r." % point, self)
+        print_d(f"Masking {point!r}", self._name)
         removed = {}
-        for item in itervalues(self):
+        for item in self.values():
             if item.mountpoint == point:
                 removed[item.key] = item
         if removed:
@@ -809,31 +812,106 @@ class FileLibrary(PicklingLibrary):
     def masked_mount_points(self):
         """List of mount points that contain masked items"""
 
-        return listkeys(self._masked)
+        return list(self._masked.keys())
 
     def get_masked(self, mount_point):
         """List of items for a mount point"""
 
-        return listvalues(self._masked.get(mount_point, {}))
+        return list(self._masked.get(mount_point, {}).values())
 
     def remove_masked(self, mount_point):
         """Remove all songs for a masked point"""
 
         self._masked.pop(mount_point, {})
 
+    def move_root(self, old_root: str, new_root: fsnative) \
+            -> Generator[None, None, None]:
+        """
+        Move the root for all songs in a given (scan) directory.
+
+        We avoid dereferencing the destination, to allow users things like:
+          1. Symlink new_path -> old_root
+          2. Move QL root to new_path
+          3. Remove symlink
+          4. Move audio files: old_root -> new_path
+
+        """
+        old_path = Path(normalize_path(old_root, canonicalise=True)).expanduser()
+        new_path = Path(normalize_path(new_root)).expanduser()
+        if not old_path.is_dir():
+            raise ValueError(f"Source {old_path!r} is not a directory")
+        if not new_path.is_dir():
+            raise ValueError(f"Destination {new_path!r} is not a directory")
+        print_d(f"{self._name}: checking {len(self.values())} item(s) for {old_path!r}")
+        missing: Set[AudioFile] = set()
+        changed = set()
+        total = len(self)
+        if not total:
+            return
+        with Task(_("Library"), _("Moving library files")) as task:
+            yield
+            for i, song in enumerate(list(self.values())):
+                task.update(i / total)
+                key = normalize_path(song.key)
+                path = Path(key)
+                if old_path in path.parents:
+                    # TODO: more Pathlib-friendly dir replacement...
+                    new_key = key.replace(str(old_path), str(new_path), 1)
+                    new_key = normalize_path(new_key, canonicalise=False)
+                    if new_key == key:
+                        print_w(f"Substitution failed for {key!r}")
+                    # We need to update ~filename and ~mountpoint
+                    song.sanitize()
+                    song.write()
+                    if self.move_song(song, new_key):
+                        changed.add(song)
+                    else:
+                        missing.add(song)
+                elif not (i % 1000):
+                    print_d(f"Not moved, for example: {key!r}")
+                if not i % 100:
+                    yield
+            self.changed(changed)
+            if missing:
+                print_w(f"Couldn't find {len(list(missing))} files: {missing}")
+        yield
+        self.save()
+        print_d(f"Done moving to {new_path!r}.")
+
+    def move_song(self, song: AudioFile, new_path: fsnative) -> bool:
+        """Updates the location of a song, without touching the file.
+
+        :returns: True if it was could be found (and moved)
+        """
+        existed = True
+        key = song.key
+        print_d(f"Moving {key!r} -> {new_path!r}")
+        try:
+            del self._contents[key]
+        except KeyError:
+            existed = False
+            # Continue - maybe it's already moved
+        song.sanitize(new_path)
+        self._contents[new_path] = song
+        return existed
+
 
 class SongFileLibrary(SongLibrary, FileLibrary):
     """A library containing song files.
     Pickles contents to disk as `FileLibrary`"""
 
     def __init__(self, name=None):
-        print_d("Initializing SongFileLibrary \"%s\"." % name)
-        super(SongFileLibrary, self).__init__(name)
+        print_d(f"Initializing {type(self)}: {name!r}")
+        super().__init__(name)
 
     def contains_filename(self, filename):
         key = normalize_path(filename, True)
         return key in self._contents
 
+    def get_filename(self, filename):
+        key = normalize_path(filename, True)
+        return self._contents.get(key)
+
     def add_filename(self, filename, add=True):
         """Add a song to the library based on filename.
 
@@ -854,7 +932,7 @@ class SongFileLibrary(SongLibrary, FileLibrary):
             if song and add:
                 self.add([song])
         else:
-            print_d("Already got file %r." % filename)
+            print_d(f"Already got file {filename!r}")
             song = self._contents[key]
 
         return song
